package net.udby.test.db.lock;

import java.io.FileOutputStream;
import java.io.IOException;
import java.util.*;
import java.util.Map.Entry;

import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFCellStyle;
import org.apache.poi.hssf.usermodel.HSSFFont;
import org.apache.poi.hssf.usermodel.HSSFRichTextString;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.hssf.util.CellRangeAddress;
import org.apache.poi.hssf.util.HSSFColor;

public class ExcelReportGenerator {
    private final HSSFWorkbook wb;
    private HSSFSheet currentSheet;
    private final Map<String, HSSFSheet> sheetMap = new HashMap<String, HSSFSheet>();
    private final short COLOR_GREEN;
    private final short COLOR_YELLOW;
    private int rowno;
    
    public ExcelReportGenerator() {
        this.wb = new HSSFWorkbook();
        this.COLOR_GREEN = HSSFColor.GREEN.index;
        this.COLOR_YELLOW = HSSFColor.YELLOW.index;
    }

    public void createSheet(String name) {
        this.currentSheet = this.wb.createSheet(name);
        this.sheetMap.put(name, this.currentSheet);
        this.rowno = 0;
    }
    
    public void driverInfo(List<String[]> driverInfo) {
        currentSheet.setColumnWidth(1, 28*256);
        HSSFRow row = currentSheet.createRow(1);
        HSSFFont fontBigBold = wb.createFont();
        fontBigBold.setFontHeightInPoints((short)12);
        fontBigBold.setBoldweight(HSSFFont.BOLDWEIGHT_BOLD);
        HSSFCellStyle style = wb.createCellStyle();
        style.setFont(fontBigBold);
        row.setRowStyle(style);
        HSSFCell cell = row.createCell(1);
        cell.setCellValue(new HSSFRichTextString("JDBC driver parameters"));
        cell.setCellStyle(style);
        currentSheet.addMergedRegion(new CellRangeAddress(1,1,1,2));
        rowno=2;
        for (String[] s2 : driverInfo) {
            HSSFRow r = currentSheet.createRow(rowno++);
            cell = r.createCell(1);
            cell.setCellValue(new HSSFRichTextString(s2[0]));
            cell = r.createCell(2);
            cell.setCellValue(new HSSFRichTextString(s2[1]));
        }
        rowno++;
    }
    
    public void writeToFile(String name) throws IOException {
        FileOutputStream fileOut = new FileOutputStream(name);
        try {
            wb.write(fileOut);
        } finally {
            fileOut.close();
        }
    }

    public void testReportFailed(int i, String message) {
        HSSFRow r = createTestRow(i);
        
        HSSFCell cell = r.createCell(2);
        cell.setCellValue(new HSSFRichTextString("failed"));
        HSSFCellStyle style = wb.createCellStyle();
        HSSFFont fontRed = wb.createFont();
        fontRed.setColor(HSSFFont.COLOR_RED);
        fontRed.setBoldweight(HSSFFont.BOLDWEIGHT_BOLD);
        fontRed.setItalic(true);
        style.setFont(fontRed);
        cell.setCellStyle(style);

        cell = r.createCell(3);
        cell.setCellValue(new HSSFRichTextString(message));
    }

    public void testReportSuccess(String msg, int i, long timens) {
        HSSFRow r = createTestRow(i);

        HSSFCell cell = r.createCell(0);
        cell.setCellValue(new HSSFRichTextString(msg));
        
        cell = r.createCell(2);
        cell.setCellValue(new HSSFRichTextString("success"));
        HSSFCellStyle style = wb.createCellStyle();
        HSSFFont fontGreen = wb.createFont();
        fontGreen.setBoldweight(HSSFFont.BOLDWEIGHT_BOLD);
        fontGreen.setColor(COLOR_GREEN);
        fontGreen.setItalic(true);
        style.setFont(fontGreen);
        cell.setCellStyle(style);

        cell = r.createCell(3);
        cell.setCellValue(timens * 1e-9);
    }

    public void testReportSkipped(int i) {
        HSSFRow r = createTestRow(i);

        HSSFCell cell = r.createCell(2);
        cell.setCellValue(new HSSFRichTextString("skipped"));
        HSSFCellStyle style = wb.createCellStyle();
        HSSFFont fontYellow = wb.createFont();
        fontYellow.setColor(COLOR_YELLOW);
        style.setFont(fontYellow);
        cell.setCellStyle(style);
    }

    private HSSFRow createTestRow(int i) {
        HSSFRow r = currentSheet.createRow(rowno+i);
        HSSFFont fontBigBold = wb.createFont();
        fontBigBold.setFontHeightInPoints((short)12);
        fontBigBold.setBoldweight(HSSFFont.BOLDWEIGHT_BOLD);
        HSSFCellStyle style = wb.createCellStyle();
        style.setFont(fontBigBold);

        HSSFCell cell = r.createCell(1);
        cell.setCellValue(new HSSFRichTextString("TEST "+i));
        cell.setCellStyle(style);
        return r;
    }

    public void showProperties(Properties properties) {
        List<Entry<Object, Object>> entries = new ArrayList<Entry<Object,Object>>(properties.entrySet());
        Collections.sort(entries, new Comparator<Entry<Object, Object>>(){
            @Override
            public int compare(Entry<Object, Object> o1, Entry<Object, Object> o2) {
                String k1 = o1.getKey().toString();
                String k2 = o2.getKey().toString();
                return k1.compareTo(k2);
            }
        });
        rowno = 2;
        for (Entry<Object, Object> entry : entries) {
            HSSFRow row = currentSheet.createRow(this.rowno++);
            HSSFCell cell = row.createCell(1);
            cell.setCellValue(new HSSFRichTextString(entry.getKey().toString()));
            cell = row.createCell(2);
            cell.setCellValue(new HSSFRichTextString(entry.getValue().toString()));
        }
    }
}
