package net.udby.test.db.lock;

import java.util.concurrent.CountDownLatch;

public abstract class Runner implements Runnable {
    protected final TestDriver driver;
    protected final ConnectionSpec cs;
    private final CountDownLatch start;
    private final CountDownLatch go;
    public static final int RUNS = 1000;
    
    /**
     * @param driver
     * @param cs
     * @param start
     * @param go
     */
    public Runner(TestDriver driver, ConnectionSpec cs, CountDownLatch start, CountDownLatch go) {
        super();
        this.driver = driver;
        this.cs = cs;
        this.start = start;
        this.go = go;
    }

    public final void run() {
        Logger.getInstance().log("started");
        try {
            cs.openConnection();
            start.countDown();
            go.await();
            Logger.getInstance().log("running");
            doWork();
        } catch (Exception e) {
            Logger.getInstance().log("failed !");
            FirstFailure.getInstance().failure(this, e);
        } finally {
            ConnectionSpec.cleanup();
            Logger.getInstance().log("done...");
        }
    }
    
    protected abstract void doWork() throws Exception;
}
